@@ -2,21 +2,17 @@ use chrono::{offset::FixedOffset, DateTime, NaiveDate, NaiveDateTime, NaiveTime}
 
 use uuid::Uuid;
 
-/// SQL values such as int, double, string timestamp
+/// SQL values such as int, double, string, timestamp
 #[derive(Debug, Clone, PartialEq)]
 pub enum Value {
     /// Literal signed long
     Long(i64),
     /// Literal floating point value
     Double(f64),
-    /// Unquoted string
-    String(String),
     /// Uuid value
     Uuid(Uuid),
     /// 'string value'
     SingleQuotedString(String),
-    /// "string value"
-    DoubleQuotedString(String),
     /// Boolean value true or false,
     Boolean(bool),
     /// Date value
@@ -36,10 +32,8 @@ impl ToString for Value {
         match self {
             Value::Long(v) => v.to_string(),
             Value::Double(v) => v.to_string(),
-            Value::String(v) => v.to_string(),
             Value::Uuid(v) => v.to_string(),
             Value::SingleQuotedString(v) => format!("'{}'", v),
-            Value::DoubleQuotedString(v) => format!("\"{}\"", v),
             Value::Boolean(v) => v.to_string(),
             Value::Date(v) => v.to_string(),
             Value::Time(v) => v.to_string(),
